% --- Source file: man/samc.resample.Rd ---
\name{BSR.sample.model}
\alias{BSR.sample.model}
\title{BSR.sample.model}
\description{Generates models from the posterior distribution estimated from BSR.samc. The .out1 file contains
 the summary for each sampled model. The .out2 file contains the estimated coefficients for each model listed in the 
 .out1 file. The .best file contains the best model among the sampled models.}
\usage{
BSR.sample.model(data, outdir, response.var, id.var, vars0, groups=NULL, 
              interactions=NULL, op=NULL)}

\arguments{
  \item{data}{See \code{\link{BSR}}.}
  \item{outdir}{See \code{\link{BSR}}.}
  \item{response.var}{See \code{\link{BSR}}.}
  \item{id.var}{See \code{\link{BSR}}.}
  \item{vars0}{See \code{\link{BSR}}.}
  \item{groups}{See \code{\link{BSR}}.}
  \item{interactions}{See \code{\link{BSR}}.}
  \item{op}{List of options. See \code{details} for all possible options.}
}
\value{
  A list of the output files created in \code{outdir} and if the input data frame was modified, then the modified data frame
  is also returned.
 
}
\details{
  
 \bold{Options list:} \cr

 The same options as in \code{\link{BSR.samc}} can be specified, except that the following options have no effect:
 \code{stepscale}, \code{lowE}, \code{maxE}, \code{scale} and \code{rho}. 
 The options \code{maxvar}, \code{maxbeta} and \code{gamma} should to be set to the same values for when the 
  \code{\link{BSR.samc}} function was called.
 There are also two additional options below for \code{op}.
  \itemize{
  \item \code{in_models} NULL or the "out_models" file from \code{\link{BSR.samc}}. The default is NULL.
  \item \code{in_hist} The "out_hist" file from \code{\link{BSR.samc}}. This must be specified.
  
  } % END: itemize

}
\seealso{\code{\link{BSR.samc}}, \code{\link{BSR.prediction}}, \code{\link{BSR}}}
\author{Faming Liang, William Wheeler, Kai Yu}
\examples{

  # Create data
  set.seed(456)
  n    <- 100
  m    <- 50
  y    <- rbinom(n, 1, 0.5)
  id   <- paste("sub_", 1:n, sep="")
  int  <- rep(1, n)
  z1   <- rbinom(n, 1, 0.5)
  z2   <- rbinom(n, 1, 0.5)
  z3   <- runif(n)
  z4   <- runif(n)
  data <- data.frame(y, id, int, z1, z2, z3, z4)

  outdir     <- "c:/temp/"
  vars0      <- "int"  # The intercept variable
  
  # Options for a small test run
  op <- list(total_iteration=100, warm=10)

  # Not run
  #ret <- BSR.sample.model(data, outdir, "y", "id", vars0, groups=NULL,
  #       interactions=NULL,  op=op)
  
}