#include "./samc.h"

void cost(x, ret, checkInt, parms)
int *x;        /* Indicator vector for groups of variables in DataX that are in the model */
RET_COST *ret; /* Return structure */
int checkInt; /* 0 or 1 to check the interactions */
PARMS *parms;
{
  int i, j, k, iter, dim, num_grp, *checkCol, *groups=NULL, insertRow, foundRow, *col_ids=NULL;
  double **datax, eps=1.0e-6, deviance; 
  double *beta, *mu, *W, *newbeta, disnorm, **A, **IA, *b;
  double *pyhat, temp;
  RET_COST *rFound, **Models=parms->Models;
  int DataNum=parms->DataNum, DataP=parms->DataP, Model_n=parms->Model_n, Model_maxn=parms->Model_maxn;
  int IntX_flag=parms->IntX_flag, **IntX=parms->IntX, *IntX_N=parms->IntX_N, TotalNcol=parms->TotalNcol, Data0X_n=parms->Data0X_n;
  int *Data0X=parms->Data0X, *GroupN=parms->GroupN, **GroupX=parms->GroupX, *indx=parms->indx;
  double **DataX=parms->DataX, *DataY=parms->DataY, GAMMA=parms->gamma, Max_Beta=parms->maxbeta;
  int *origGroups=NULL, *ptri;

  parms->newModel = 0;
  origGroups = ivector(1, DataP);

  if (DEBUG) printf("Begin cost\n");

  /* Initialize the return flag */
  ret->converged = -9;
  ret->value     = 1.0e+110;
  ret->deviance  = 1.0e+110;
  pyhat = ret->Yhat;
  for (i = 1; i<=DataNum; i++) pyhat[i] = -1.0;

  /* Determine the number of groups in the model */
  for(i=1, num_grp=0; i<=DataP; i++) {
    if (x[i]) num_grp++;
  }
  ret->num_grp = num_grp;
  /* Put these groups in a vector. The vector is already re-sorted. Original groups are not sorted. */
  if (num_grp) {
    groups = ret->groups;
    for(i=1, k=1; i<=DataP; i++) {
      if (x[i]) {
        groups[k] = i;
        origGroups[k] = indx[i];
        k++;
      }
    }
 
    /* The vector origGroups must be arranged so the element is increasing order */
    for (i=1; i<=num_grp-1; i++) {
      for (j=i+1; j<=num_grp; j++) {
	if (origGroups[i] > origGroups[j]) {
	  k = origGroups[i];
	  origGroups[i] = origGroups[j];
	  origGroups[j] = k;
	}
      }
    } 
  } /* end if */


  /* Check the interactions , for the moment, don't check interaction*/
  if ((checkInt) && (IntX_flag)) {
    for (i=1; i<=DataP; i++) {
      k = IntX_N[i];
      if ((x[i]) && (k)) {
        /* Group i is in the model, and it has other groups that must be in the model. Check its associated groups */
        ptri = IntX[i];
        for (j=0; j<k; j++) {
          if (!x[ptri[j]]) {
            ret->converged = -8;
	    free_ivector(origGroups, 1, DataP);
            return;
          }
        }
      }
    }
  }

  /* Determine if this model was already run */
  if (DEBUG) printf("Begin find_row, Model_n=%d, Model_maxn=%d\n", Model_n, Model_maxn);
  foundRow = find_row(origGroups, num_grp, &insertRow, parms);
  if (DEBUG) printf("End find_row, row = %d\n", foundRow);
  if (foundRow) {
    /* Model was already run. Results are in Models[foundRow] */
    rFound         = Models[foundRow];
    ret->value     = rFound->value;
    ret->deviance  = rFound->deviance;
    ret->dim       = rFound->dim;
    ret->num_grp   = rFound->num_grp;
    ret->converged = rFound->converged; 
    copy_ivec(ret->groups, rFound->groups, 1, rFound->num_grp); 
    copy_ivec(ret->col_ids, rFound->col_ids, 1, rFound->dim);
    copy_dvec(ret->beta, rFound->beta, 1, rFound->dim);
    free_ivector(origGroups, 1, DataP);

    parms->newModel = 1;

    /* Compute probs 
    post_prob(foundRow, ret->Yhat, parms);
    */
    return;
  }
  


  /* Vector to check that a column does not get added twice */
  checkCol = ivector(1, TotalNcol);
  for (i=1; i<=TotalNcol; i++) checkCol[i] = 0;
  if (Data0X_n) {
    for (j=1; j<=Data0X_n; j++) checkCol[Data0X[j]] = 1;
  }

  /* Get the correct dimension. Check that a column is not added twice */
  dim = Data0X_n;
  for(i=1; i<=DataP; i++) {
    if (x[i]) {
      for (j=1; j<=GroupN[i]; j++) {
        k = GroupX[i][j];
        if (!checkCol[k]) {
          dim++;
          checkCol[k] = 1;
        }
      }
    }
  }
  
  datax=dmatrix(1,DataNum,1,dim);
  W=dvector(1,DataNum);
  A=dmatrix(1,dim,1,dim);
  IA=dmatrix(1,dim,1,dim);
  b=dvector(1,dim);
  newbeta=dvector(1,dim);
  mu=dvector(1,DataNum);

  beta = ret->beta;  /* No need to keep re-allocating memory for beta */
  col_ids = ret->col_ids;

  if (DEBUG) printf("Set up data matrix, dim=%d, DataNum=%d, DataP=%d, Data0X_n=%d\n", dim, DataNum, DataP, Data0X_n);

  /* Add all variables in this model using the checkCol vector */
 
  for(j=1, k=0; j<=TotalNcol; j++) {
    if(checkCol[j]) {
      k++; 
      col_ids[k] = j;
      for(i=1; i<=DataNum; i++) datax[i][k]=DataX[i][j];
    }
  }

  free_ivector(checkCol, 1, TotalNcol);


  /*** Initial value   ***/
  if (DEBUG) {
    printf("Compute initial estimates\n");
    printf("Compute X`X\n");
  }
  for(i=1; i<=dim; i++){
     for(j=1; j<=dim; j++){
        A[i][j]=0.0;
        for(k=1; k<=DataNum; k++) A[i][j]+=datax[k][i]*datax[k][j];
      }
    }
  

  for(i=1; i<=dim; i++) A[i][i]+=1.0e-20;
  if (DEBUG) printf("Compute matrix inverse, dimension = %d\n", dim);
  matrix_inverse(A,IA,dim);

  if (DEBUG) printf("Compute X`(2Y-1)\n");
  for(j=1; j<=dim; j++){
        b[j]=0.0;
        for(k=1; k<=DataNum; k++) b[j]+=datax[k][j]*(2.0*DataY[k]-1.0);
       }

   if (DEBUG) printf("Compute beta0\n");
     for(i=1; i<=dim; i++){
        beta[i]=0.0;
        for(j=1; j<=dim; j++) beta[i]+=IA[i][j]*b[j];
      }

  disnorm=1.0; iter=0; 

  ret->dim       = dim;
  if (DEBUG) printf("Begin while loop\n");
  while(disnorm>eps && iter<MAXIT_COST){     
    /* Check the betas */
    for (j=1; j<=dim; j++) {
      if (fabs(beta[j]) > Max_Beta) 
	{
	  ret->converged = -7;
    	  goto ABC;
	}
    }
 
    for(k=1; k<=DataNum; k++){
        mu[k]=0.0;
        for(j=1; j<=dim; j++) mu[k]+=datax[k][j]*beta[j];
        mu[k]=1.0/(1.0+exp(-mu[k]));
        W[k]=mu[k]*(1-mu[k]);
      }

    for(i=1; i<=dim; i++)
       for(j=1; j<=dim; j++){
           A[i][j]=0.0;
           for(k=1; k<=DataNum; k++) A[i][j]+=W[k]*datax[k][i]*datax[k][j]; 
        }
  
    for(i=1; i<=dim; i++) A[i][i]+=1.0e-20;
  
    matrix_inverse(A,IA,dim); 
    
    for(j=1; j<=dim; j++){
        b[j]=0.0;
        for(k=1; k<=DataNum; k++) b[j]+=datax[k][j]*(DataY[k]-mu[k]);
       }
        
    for(i=1; i<=dim; i++){
        newbeta[i]=beta[i];
        for(j=1; j<=dim; j++) newbeta[i]+=IA[i][j]*b[j];
      }

   
    for(disnorm=0.0, i=1; i<=dim; i++){
        disnorm+=(newbeta[i]-beta[i])*(newbeta[i]-beta[i]);
        beta[i]=newbeta[i];
      }
    disnorm=sqrt(disnorm);
    iter++;
  
   }
  
  if (DEBUG) printf("End while loop, iter=%d, disnorm=%g\n", iter, disnorm);

  /* not converge, not count */

  if(disnorm>eps){
    ret->converged = -8;
    
    goto ABC;
  
    }


   /*print_dVec(beta, dim, "beta");*/

     /* calculate deviance and also predicted probabilities P(Y=1 | X) */
     for(deviance=0.0,k=1; k<=DataNum; k++){
       mu[k]=0.0;
       for(j=1; j<=dim; j++) mu[k]+=datax[k][j]*beta[j];
    
       /* For predicted probabilities */
       temp = exp(mu[k]);
       pyhat[k] = temp/(1.0 + temp); 

       /*       mu[k]=1.0/(1.0+exp(-mu[k]));*/
       if(DataY[k]==1) deviance = deviance - MYLOG(pyhat[k]);     
       else deviance = deviance - MYLOG(1-pyhat[k]);
     }
     deviance*=2.0;  /*-2*log likelihood */

     /* question here */ 
     if(deviance<=0.1)
     { 
        deviance = 0.1;
     }

     /*ret->value = 0.5*(deviance+dim*PenaltyC);*/
     /* this is similar to EBIC, the samller, the better fit*/
     ret->value = 0.5*(deviance + dim * log(0.0 + DataNum) + 
		     2.0 * GAMMA * num_grp * log(1.0 * DataP));
     ret->deviance  = deviance;
     ret->dim       = dim;
     ret->converged = 1;
   

ABC:
   /* Save the model. Use the  original groups */

   copy_ivec(ret->groups, origGroups, 1, num_grp); 

   insert_model(insertRow, ret, parms);
   
   free_ivector(origGroups, 1, DataP);

   free_dmatrix(datax,1,DataNum,1,dim);
   free_dvector(W,1,DataNum);
   free_dmatrix(A,1,dim,1,dim);
   free_dmatrix(IA,1,dim,1,dim);
   free_dvector(b,1,dim);
   free_dvector(newbeta,1,dim);
   free_dvector(mu,1,DataNum);

   if (DEBUG) printf("End cost, converged=%d, value=%g, deviance=%g\n", ret->converged, ret->value, ret->deviance);

   return;
  }


